/*
 * Copyright 2009 Cristian Vasile Mocanu (cvmocanu@gmail.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.mocanu.intellij.cic;

import com.intellij.openapi.module.Module;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.roots.ProjectFileIndex;
import com.intellij.openapi.roots.ProjectRootManager;
import com.intellij.openapi.util.TextRange;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.psi.*;
import com.intellij.psi.search.GlobalSearchScope;
import com.intellij.psi.search.PsiShortNamesCache;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;

public class ReferenceToClassesFromComments extends PsiReferenceBase.Poly<PsiComment> {
    private String potentialClassName;

    private Module module;
    private boolean moduleInitialized = false;

    public ReferenceToClassesFromComments(PsiComment psiElement, String potentialClassName) {
        super(psiElement);
        this.potentialClassName = potentialClassName;
    }

    public ReferenceToClassesFromComments(PsiComment element, boolean soft, String potentialClassName) {
        super(element, soft);
        this.potentialClassName = potentialClassName;
    }

    public ReferenceToClassesFromComments(PsiComment element, TextRange range, boolean soft, String potentialClassName) {
        super(element, range, soft);
        this.potentialClassName = potentialClassName;
    }

    public Object[] getVariants() {
        return null;
    }

    @NotNull
    public ResolveResult[] multiResolve(boolean incompleteCode) {
        List<ResolveResult> result = new ArrayList<ResolveResult>();

        PsiClass[] psiClasses = getClasses();
        if (psiClasses == null || psiClasses.length == 0) {
            return ResolveResult.EMPTY_ARRAY;
        }

        for (PsiClass psiClass : psiClasses) {
            result.add(
                    new PsiElementResolveResult(psiClass)
            );
        }

        return result.toArray(new ResolveResult[result.size()]);
    }

    @Nullable
    private Module getModule(Project project) {
        if (!moduleInitialized) {
            module = getModuleForPsiElement(myElement, project);
            moduleInitialized = true;
        }

        return module;
    }

    @Nullable
    private static Module getModuleForPsiElement(PsiElement psiElement, Project project) {
        ProjectFileIndex projectfileindex = ProjectRootManager.getInstance(project).getFileIndex();
        VirtualFile file = psiElement.getContainingFile().getVirtualFile();
        if (file == null) {
            return null;
        }

        return projectfileindex.getModuleForFile(file);
    }

    private PsiClass[] getClasses() {
        Project project = myElement.getProject();

        Module module = getModule(project);

        GlobalSearchScope globalsearchscope;
        if (module != null) {
            globalsearchscope = GlobalSearchScope.moduleWithDependenciesAndLibrariesScope(module);
        } else {
            globalsearchscope = GlobalSearchScope.projectScope(project);
        }

        JavaPsiFacade javaPsiFacade = JavaPsiFacade.getInstance(project);
        if (potentialClassName.indexOf('.') != -1) {

            return javaPsiFacade.findClasses(potentialClassName, globalsearchscope);
        } else {
            PsiShortNamesCache cache = javaPsiFacade.getShortNamesCache();

            return cache.getClassesByName(potentialClassName, globalsearchscope);
        }
    }
}
